// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>

#include "ApiBridges.h"

int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);

    QtOpenAPI::ColorsApi colorsApiInstance;
    ForeignColorsAPI::s_singletonInstance = &colorsApiInstance;

    QtOpenAPI::UsersApi usersApiInstance;
    ForeignUsersAPI::s_singletonInstance = &usersApiInstance;

    QQmlApplicationEngine engine;

    QObject::connect(
            &engine, &QQmlApplicationEngine::objectCreationFailed, &app,
            []() { QCoreApplication::exit(-1); }, Qt::QueuedConnection);

    engine.loadFromModule("ColorPalette", "Main");

    return app.exec();
}
